@@ -1,6 +1,7 @@
 package js
 
 import (
+	"bytes"
 	"errors"
 	"fmt"
 	"io"
@@ -862,7 +863,7 @@ func (p *Parser) parseFuncParams(in string) (params Params) {
 	p.next()
 
 	// mark undeclared vars as arguments in `function f(a=b){var b}` where the b's are different vars
-	p.scope.MarkArguments()
+	p.scope.MarkFuncArgs()
 	return
 }
 
@@ -975,23 +976,21 @@ func (p *Parser) parseAnyClass(inExpr bool) (classDecl *ClassDecl) {
 			break
 		}
 
-		method, definition := p.parseClassElement()
-		if method != nil {
-			classDecl.Methods = append(classDecl.Methods, method)
-		} else {
-			classDecl.Definitions = append(classDecl.Definitions, definition)
-		}
+		classDecl.List = append(classDecl.List, p.parseClassElement())
 	}
 	return
 }
 
-func (p *Parser) parseClassElement() (method *MethodDecl, definition FieldDefinition) {
-	method = &MethodDecl{}
+func (p *Parser) parseClassElement() ClassElement {
+	method := &MethodDecl{}
 	var data []byte
 	if p.tt == StaticToken {
 		method.Static = true
 		data = p.data
 		p.next()
+		if p.tt == OpenBraceToken {
+			return ClassElement{StaticBlock: p.parseBlockStmt("class static block")}
+		}
 	}
 	if p.tt == MulToken {
 		method.Generator = true
@@ -1017,7 +1016,7 @@ func (p *Parser) parseClassElement() (method *MethodDecl, definition FieldDefini
 		p.next()
 	}
 
-	isFieldDefinition := false
+	isField := false
 	if data != nil && p.tt == OpenParenToken {
 		method.Name.Literal = LiteralExpr{IdentifierToken, data}
 		if method.Async || method.Get || method.Set {
@@ -1029,27 +1028,25 @@ func (p *Parser) parseClassElement() (method *MethodDecl, definition FieldDefini
 		}
 	} else if data != nil && (p.tt == EqToken || p.tt == SemicolonToken || p.tt == CloseBraceToken) {
 		method.Name.Literal = LiteralExpr{IdentifierToken, data}
-		isFieldDefinition = true
-	} else if data == nil && p.tt == PrivateIdentifierToken {
+		isField = true
+	} else if (data == nil || bytes.Equal(data, []byte("static"))) && p.tt == PrivateIdentifierToken {
 		method.Name.Literal = LiteralExpr{p.tt, p.data}
 		p.next()
-		isFieldDefinition = true
+		isField = true
 	} else {
-		method.Name = p.parsePropertyName("method definition")
-		if data == nil && p.tt != OpenParenToken {
-			isFieldDefinition = true
+		method.Name = p.parsePropertyName("method or field definition")
+		if (data == nil || bytes.Equal(data, []byte("static"))) && p.tt != OpenParenToken {
+			isField = true
 		}
 	}
 
-	if isFieldDefinition {
-		// FieldDefinition
-		definition.Name = method.Name
+	if isField {
+		var init IExpr
 		if p.tt == EqToken {
 			p.next()
-			definition.Init = p.parseExpression(OpAssign)
+			init = p.parseExpression(OpAssign)
 		}
-		method = nil
-		return
+		return ClassElement{Field: Field{Static: method.Static, Name: method.Name, Init: init}}
 	}
 
 	parent := p.enterScope(&method.Body.Scope, true)
@@ -1062,7 +1059,7 @@ func (p *Parser) parseClassElement() (method *MethodDecl, definition FieldDefini
 
 	p.await, p.yield = parentAwait, parentYield
 	p.exitScope(parent)
-	return
+	return ClassElement{Method: method}
 }
 
 func (p *Parser) parsePropertyName(in string) (propertyName PropertyName) {
@@ -1491,7 +1488,7 @@ func (p *Parser) parseArrowFuncBody() (list []IStmt) {
 	p.next()
 
 	// mark undeclared vars as arguments in `function f(a=b){var b}` where the b's are different vars
-	p.scope.MarkArguments()
+	p.scope.MarkFuncArgs()
 
 	if p.tt == OpenBraceToken {
 		parentInFor := p.inFor
@@ -1854,7 +1851,7 @@ func (p *Parser) parseExpressionSuffix(left IExpr, prec, precLeft OpPrec) IExpr
 			if p.tt != PrivateIdentifierToken {
 				p.tt = IdentifierToken
 			}
-			left = &DotExpr{left, LiteralExpr{p.tt, p.data}, exprPrec}
+			left = &DotExpr{left, LiteralExpr{p.tt, p.data}, exprPrec, false}
 			p.next()
 			if precLeft < OpMember {
 				precLeft = OpCall
@@ -1874,7 +1871,7 @@ func (p *Parser) parseExpressionSuffix(left IExpr, prec, precLeft OpPrec) IExpr
 			}
 			parentInFor := p.inFor
 			p.inFor = false
-			left = &IndexExpr{left, p.parseExpression(OpExpr), exprPrec}
+			left = &IndexExpr{left, p.parseExpression(OpExpr), exprPrec, false}
 			p.inFor = parentInFor
 			if !p.consume("index expression", CloseBracketToken) {
 				return nil
@@ -1893,7 +1890,7 @@ func (p *Parser) parseExpressionSuffix(left IExpr, prec, precLeft OpPrec) IExpr
 			}
 			parentInFor := p.inFor
 			p.inFor = false
-			left = &CallExpr{left, p.parseArguments()}
+			left = &CallExpr{left, p.parseArguments(), false}
 			precLeft = OpCall
 			p.inFor = parentInFor
 		case TemplateToken, TemplateStartToken:
@@ -1919,21 +1916,24 @@ func (p *Parser) parseExpressionSuffix(left IExpr, prec, precLeft OpPrec) IExpr
 			}
 			p.next()
 			if p.tt == OpenParenToken {
-				left = &OptChainExpr{left, &CallExpr{nil, p.parseArguments()}}
+				left = &CallExpr{left, p.parseArguments(), true}
 			} else if p.tt == OpenBracketToken {
 				p.next()
-				left = &OptChainExpr{left, &IndexExpr{nil, p.parseExpression(OpExpr), OpCall}}
+				left = &IndexExpr{left, p.parseExpression(OpExpr), OpCall, true}
 				if !p.consume("optional chaining expression", CloseBracketToken) {
 					return nil
 				}
 			} else if p.tt == TemplateToken || p.tt == TemplateStartToken {
 				template := p.parseTemplateLiteral(precLeft)
-				left = &OptChainExpr{left, &template}
+				template.Prec = OpCall
+				template.Tag = left
+				template.Optional = true
+				left = &template
 			} else if IsIdentifierName(p.tt) {
-				left = &OptChainExpr{left, &LiteralExpr{IdentifierToken, p.data}}
+				left = &DotExpr{left, LiteralExpr{IdentifierToken, p.data}, OpCall, true}
 				p.next()
 			} else if p.tt == PrivateIdentifierToken {
-				left = &OptChainExpr{left, &LiteralExpr{p.tt, p.data}}
+				left = &DotExpr{left, LiteralExpr{p.tt, p.data}, OpCall, true}
 				p.next()
 			} else {
 				p.fail("optional chaining expression", IdentifierToken, OpenParenToken, OpenBracketToken, TemplateToken)
@@ -2198,7 +2198,7 @@ func (p *Parser) parseParenthesizedExpressionOrArrowFunc(prec OpPrec, async []by
 				args.List = append(args.List, Arg{Value: rest, Rest: true})
 			}
 			left = p.scope.Use(async)
-			left = &CallExpr{left, args}
+			left = &CallExpr{left, args, false}
 			precLeft = OpCall
 		} else {
 			// parenthesized expression
